package pathfinder.test;

import java.beans.Visibility;
import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;
import java.awt.event.*;
import java.util.Random;
import java.text.DecimalFormat;
import pathfinder.*;
import java.awt.image.*;

public class GUI extends JPanel {
    
    private final CollisionDetector cd;
    private final int width, height;
    private final AStar aStar;
    private ArrayList<Node> path = null;
    private Random rand = new Random();
    private final PFVisualization visualization;
    
    public GUI(final AStar aStar, CollisionDetector cd, int width, int height){
        JFrame frame = new JFrame("Pathfinder test");
        this.width = width;
        this.height = height;
        this.visualization = new PFVisualization(aStar, cd);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.cd = cd;
        this.aStar = aStar;
        setPreferredSize(new Dimension(512, 512));
        
        
        this.setBackground(Color.black);
        
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        panel.add(this, BorderLayout.CENTER);
        JButton button = new JButton("Iterate");
        button.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                
                int status = aStar.iterate();
                if(status == aStar.COMPLETED){
                    path = GUI.this.aStar.getPath();
                }
                GUI.this.repaint();
            }
        });
        JButton resetButton = new JButton("Reset");
        resetButton.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                int srcX = 1+rand.nextInt(aStar.width-2);
                int srcY = 1+rand.nextInt(aStar.height-2);
                int dstX = 1+rand.nextInt(aStar.width-2);
                int dstY = 1+rand.nextInt(aStar.height-2);
                setSrcDst(srcX, srcY, dstX, dstY);
                repaint();
                path=null;
            }
        });
        JToolBar bar = new JToolBar();
        bar.add(button);
        bar.add(resetButton);
        
        panel.add(bar, BorderLayout.NORTH);
        
        frame.add(panel);
        frame.pack();
        frame.setVisible(true);
    }
    
    float maxCost = 0;
    
    public void setSrcDst(int srcX, int srcY, int dstX, int dstY){
        aStar.setup(srcX, srcY, dstX, dstY, cd);
    }
    
    /*public final static int colorCount = 16;
    public final static Color[] colors = new Color[colorCount];
    static {
        for(int i = 0; i < colorCount; i++){
            colors[i] = new Color(1f*i/colorCount, 1f-1f*i/colorCount, 0f);
        }
    }
    
    public final static Color 
            closed = new Color(0,128,255, 64),
            open = new Color(0,255,255, 64),
            pathColor = new Color(255, 255, 128),
            blockedColor = Color.black,
            src = Color.GREEN,
            dst = Color.magenta;
    */
    protected void paintComponent(Graphics g){
        super.paintComponent(g);
        visualization.draw(g, getWidth(), getHeight());
        /*for(int i = 0; i < aStar.getNodeCount(); i++){
            Node node = aStar.getNode(i);
            float cost = cd.getCost(node.x, node.y);
            drawNode(g, node, colors[(int)((colorCount-1)*cost/maxCost)]);
            
            boolean blocked = cd.isBlocked(node.x, node.y);
            if(blocked){
                drawNode(g, node, blockedColor);
            }
            if(node.isInClosedList()){
                drawNode(g, node, closed);
            }
            if(node.isInOpenList()){
                drawNode(g, node, open);
            }
        }
        if(path != null){
            for(Node node : path){
                drawNode(g, node, pathColor);
            }
        }
        Node node;
        if((node=aStar.getSrc()) != null){
            drawNode(g, node, Color.yellow);
        }
        if((node=aStar.getDst()) != null){
            drawNode(g, node, Color.GREEN);
        }*/
        
        //STRING DRAWING
        /*g.setColor(Color.black);
        int xSize = getWidth()/width;
        int ySize = getHeight()/height;        
        for(Node node : aStar.nodes){
            if(!cd.isBlocked(node.x, node.y)){
                g.drawString("f="+(int)node.f, node.x*xSize, node.y*ySize+10);
                g.drawString(""+(int)node.g+" "+(int)node.h, node.x*xSize, node.y*ySize+20);
            }
        }*/
        
    }
    
    protected void drawNode(Graphics g, Node node, Color c){
        int xSize = getWidth()/width;
        int ySize = getHeight()/height;
        g.setColor(c);
        g.fillRect(node.x*xSize, node.y*ySize, xSize, ySize);
    }
    
}
