/*
 * MaskCollisionDetector.java
 *
 * Created on July 10, 2007, 2:23 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package pathfinder.test;

import java.util.Random;
import pathfinder.*;

/**
 *
 * @author ask
 */
public class MaskCollisionDetector implements CollisionDetector {

    public final int width, height;
    final boolean[] tiles;
    final int maskSize;
    final int[] xMask, yMask;
    final Random rand;
    
    public MaskCollisionDetector(){
        this(16);
    }

    public MaskCollisionDetector(int emptyRatio){
        this(64, 64, 0L, emptyRatio);
    }
    
    public MaskCollisionDetector(int width, int height, long seed, int emptyRatio){
        //this(256, 256, 0, new int[]{0}, new int[]{0}, 2);
        this(width, height, seed, new int[]{-1,-1,-1, 0, 0, 0, 1, 1, 1}, new int[]{-1, 0, 1,-1, 0, 1,-1, 0, 1}, emptyRatio);
    }
        
    /** Creates a new instance of MaskCollisionDetector */
    public MaskCollisionDetector(int width, int height, long seed, int[] xMask, int[] yMask, int emptyRatio) {
        this.width = width;
        this.height = height;
        this.tiles = new boolean[width*height];
        this.rand = new Random(seed);
        this.maskSize = xMask.length;
        if(yMask.length != maskSize)
            throw new RuntimeException("Weird mask size!");
        this.xMask = xMask;
        this.yMask = yMask;
        if(emptyRatio != 0){
            for(int i = 0; i < tiles.length; i++){
                tiles[i] = rand.nextInt(emptyRatio)==0;
            }
        }
        for(int i = 0; i < width; i++){
            tiles[i] = true; //top
            tiles[tiles.length-1-i] = true; //bottom
        }
        System.out.println("width "+width+" height "+height);
        for(int i = 0; i < height; i++){
            tiles[width*i] = true; //left
            tiles[width*(i+1)-1] = true; //right
        }
    }
    
    public boolean isBlocked(int x, int y){
        //return tiles[x+y*width];
        boolean blocked = false;
        blocked |= tiles[x+y*width];
        for(int i = 0; i < maskSize && !blocked; i++){
            blocked |= tiles[x+xMask[i] + (y + yMask[i])*width];
        }
        return blocked;
        //return false;
    }
    
    public float getCost(int x, int y){
        return 1f;
    }
    
    public int getWidth(){
        return width;
    }
    
    public int getHeight(){
        return height;
    }
    
}
