/*
 * Visualization.java
 *
 * Created on July 11, 2007, 2:53 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package pathfinder.test;

import pathfinder.*;
import java.awt.*;
import java.awt.Image.*;
import java.util.ArrayList;

/**
 *
 * @author ask
 */
public class PFVisualization {
  
    public final int colorCount = 16;
    public final Color[] colors = new Color[colorCount];
    
    public final static Color 
            closed = new Color(0,128,255, 64),
            open = new Color(0,255,255, 64),
            pathColor = new Color(255, 255, 128),
            blockedColor = Color.black,
            src = Color.GREEN,
            dst = Color.magenta;

    
    float maxCost = 0;
    private AStar aStar;
    private PathClient client;
    private ArrayList<Node> path = null;
    
    public PFVisualization(AStar aStar, PathClient client){
        this(aStar, client, Color.GREEN, Color.RED);
    }
    
    public PFVisualization(AStar aStar, PathClient client, Color cheap, Color expensive){
        this.aStar = aStar;
        this.client = client;
        
        for(int i = 0; i < aStar.width; i++){
            for(int j = 0; j < aStar.height; j++){
                float cost = client.getCost(i,j);
                if(cost > maxCost){
                    maxCost = cost;
                }
            }
        }
        int eR = expensive.getRed(), eG = expensive.getGreen(), eB = expensive.getBlue();
        int cR = cheap.getRed(), cG = cheap.getGreen(), cB = cheap.getBlue();
        
        for(int i = 0; i < colorCount; i++){
            int r = cR + i * (eR - cR)/colorCount;
            int g = cG + i * (eG - cG)/colorCount;
            int b = cB + i * (eB - cB)/colorCount;
            
            colors[i] = new Color(r,g,b);
        }
    }
    
    public void draw(Graphics g, int width, int height){
        xSize = width/aStar.width;
        ySize = height/aStar.height;
        for(int i = 0; i < aStar.getNodeCount(); i++){
            Node node = aStar.getNode(i);
            float cost = client.getCost(node.x, node.y);
            drawNode(g, node, colors[(int)((colorCount-1)*cost/maxCost)]);
            
            boolean blocked = client.isBlocked(node.x, node.y);
            if(blocked){
                drawNode(g, node, blockedColor);
            }
            if(node.isInClosedList()){
                drawNode(g, node, closed);
            }
            if(node.isInOpenList()){
                drawNode(g, node, open);
            }
        }
        if(path != null){
            for(Node node : path){
                drawNode(g, node, pathColor);
            }
        }
        Node node;
        if((node=aStar.getSrc()) != null){
            drawNode(g, node, Color.yellow);
        }
        if((node=aStar.getDst()) != null){
            drawNode(g, node, Color.GREEN);
        }
        
        path = aStar.getPath();
        for(Node n : path){
            drawNode(g, n, pathColor);
        }
        
    }
    
    private int xSize, ySize;
    
    public void drawNode(Graphics g, Node node, Color c){
        g.setColor(c);
        g.fillRect(node.x*xSize, node.y*ySize, xSize, ySize);
    }    
    
}
